import os.path

__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__commit__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
]


try:
    base_dir = os.path.dirname(os.path.abspath(__file__))
except NameError:
    base_dir = None


__title__ = "pollyglot"
__summary__ = "Small example datasets of annotated vocalizations"
__uri__ = "https://github.com/NickleDave/pollyglot"

__version__ = "0.1.1"

if base_dir is not None and os.path.exists(os.path.join(base_dir, ".commit")):
    with open(os.path.join(base_dir, ".commit")) as fp:
        __commit__ = fp.read().strip()
else:
    __commit__ = None

__author__ = "David Nicholson"
__email__ = "nicholdav@gmail.com"

__license__ = "BSD"
__copyright__ = "2019 %s" % __author__



"""script that generates the Pollyglot data repository"""
import argparse
import os
from pathlib import Path
import pkg_resources
import shutil
import tarfile
import zipfile

import rarfile

from .datasets import dataset_dict, dataset_names
from .urlcopy import urlcopy

HERE = Path(__file__).parent

# target paths
DOWNLOADS_DIR = HERE.joinpath('../../results/downloads')
TARGZBALLS_DIR = HERE.joinpath('../../results/targzballs')


tarprep_funcs = {}
for tarprep_func in pkg_resources.iter_entry_points(group='pollyglot.tarprep'):
    tarprep_funcs.update(
        {tarprep_func.name: tarprep_func.load()}
    )


def clean():
    downloads_dir_dirs = [d for d in DOWNLOADS_DIR.iterdir() if d.is_dir()]
    if downloads_dir_dirs:
        for downloads_dir in downloads_dir_dirs:
            shutil.rmtree(downloads_dir)

    targzballs_dir_dirs = [d for d in TARGZBALLS_DIR.iterdir() if d.is_dir()]
    if targzballs_dir_dirs:
        for targzballs_dir in targzballs_dir_dirs:
            shutil.rmtree(targzballs_dir)


def makedirs_downloads():
    for dataset_name in dataset_names:
        download_dir = DOWNLOADS_DIR / f"{dataset_name}"
        download_dir.mkdir()


def download():
    for dataset_name, repo_url_dict in dataset_dict.items():
        print(
            f'downloading data for {dataset_name}'
        )
        local_repo_path = DOWNLOADS_DIR.joinpath(dataset_name)
        prefixes = []

        if 'data_url' in repo_url_dict:
            prefixes.append('data')
        elif 'audio_url' in repo_url_dict and 'annot_url' in repo_url_dict:
            prefixes.append('audio')
            prefixes.append('annot')

        for prefix in prefixes:
            url = repo_url_dict[f'{prefix}_url']
            filename = repo_url_dict[f'{prefix}_filename']
            dst = local_repo_path.joinpath(filename)
            urlcopy(url, dst=str(dst))

            print(
                f'extracting data for {dataset_name}'
            )

            if dst.suffixes[-2:] == ['.tar', '.gz']:
                tar = tarfile.open(str(dst))
                tar.extractall(str(local_repo_path))
            elif dst.suffixes[-1:] == ['.zip']:
                with zipfile.ZipFile(str(dst), 'r') as zip_ref:
                    zip_ref.extractall(str(local_repo_path))
            elif dst.suffixes[-1:] == ['.rar']:
                with rarfile.RarFile(str(dst), 'r') as rar_ref:
                    rar_ref.extractall(str(local_repo_path))


def make_targz(targz_filename, source_dir, open_as="w:gz"):
    with tarfile.open(targz_filename, open_as) as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))


def targzball():
    for dataset_name in dataset_names:
        func_key = dataset_name.replace('-', '_')
        tarprep_func =tarprep_funcs[func_key]

        data_dir = DOWNLOADS_DIR.joinpath(dataset_name)
        targz_dir = TARGZBALLS_DIR.joinpath(dataset_name)
        tarprep_func(src=data_dir, dst=targz_dir)

        targz = TARGZBALLS_DIR / f'{dataset_name}.tar.gz'
        make_targz(targz_filename=targz, source_dir=targz_dir)


def all():
    makedirs_downloads()
    download()
    targzball()


def make(command):
    """a function that acts kind of like Unix make
    except that you don't have to do backflips to get it
    to work on a whole directory
    """
    if command == 'clean':
        clean()
    elif command == 'all':
        all()
    elif command == 'makedirs_downloads':
        makedirs_downloads()
    elif command == 'download':
        download()
    elif command == 'targzball':
        targzball()
    else:
        raise ValueError(
            f'unknown command: {command}'
        )


def get_argparser():
    DESCRIPTION = 'command-line interface for script that generates Pollyglot dataset'
    CHOICES = [
        'clean',
        'all',
        'makedirs_downloads',
        'download',
        'targzball',
    ]

    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument('command', type=str, metavar='command',
                        choices=CHOICES,
                        help="Command to run, valid options are:\n"
                             f"{CHOICES}\n")
    return parser


def main():
    parser = get_argparser()
    args = parser.parse_args()
    make(args.command)


if __name__ == '__main__':
    main()



"""

"""
from math import ceil

import requests
from tqdm import tqdm

CHUNK_SIZE = 1024 * 10


def urlcopy(url, dst, stream=True, chunk_size=CHUNK_SIZE):
    """copy a file from a url to a local destination

    Parameters
    ----------
    url : str
        url that points to a data file, e.g. a .tar.gz or .zip
    dst : str
        destination, path to where file should be downloaded
    stream : bool
        if True, open url in streaming mode. Default is True.
    chunk_size : int
        number of bytes to read into memory from stream at once.
        Default is 10240.

    Returns
    -------

    """
    response = requests.get(url, stream=stream)
    content_length = int(response.headers['content-length'])

    num_chunks = int(
        ceil(content_length / chunk_size)
    )

    iter_content = response.iter_content(CHUNK_SIZE)
    if response.status_code == 200:
        with open(dst, 'wb') as fp:
            for chunk in tqdm(iter_content, total=num_chunks):
                fp.write(chunk)
    else:
        raise ValueError(
            f'was unable to request content at {url},\n'
            f'got status code {response.status_code} instead of '
            'status code 200 (HTTP OK response)'
        )


"""module that loads dataset names,
so that user can see what datasets are available.
"""
import json
from pathlib import Path

import yaml


HERE = Path(__file__).parent
DATASETS_YAML = HERE.joinpath('datasets.yaml')
with DATASETS_YAML.open() as f:
    dataset_dict = yaml.safe_load(f)

dataset_names = [k for k in dataset_dict.keys()]


def show():
    print(json.dumps(dataset_dict, indent=1))



from .__about__ import (
    __author__,
    __commit__,
    __copyright__,
    __email__,
    __license__,
    __summary__,
    __title__,
    __uri__,
    __version__,
)

from . import datasets
from . import make
from . import tarprep
from . import urlcopy

from .fetchfuncs import fetch



"""functions that prepare .tar.gz files from downloaded data"""
import shutil


def cbin_notmat(src, dst):
    """creates a tar from the cbin-notmat directory

    Parameters
    ----------
    src : pathlib.Path
        source; a sub-directory in ./results/downloads
    dst : pathlib.Path
        destination; a sub-directory in ./results/targzballs

    Returns
    -------
    None
    """
    SUBDIR = '032212'
    NUM_TO_USE = 20
    exts = ['.cbin', '.not.mat', '.rec']
    exts_files_map = {}
    for ext in exts:
        files_this_ext = sorted(list(
            src.joinpath(SUBDIR).glob(f'*{ext}')
        ))
        files_this_ext = files_this_ext[:NUM_TO_USE]
        exts_files_map[ext] = files_this_ext
    dst.mkdir()
    for _, files in exts_files_map.items():
        for src_file in files:
            dst_file = dst / src_file.name
            shutil.copy(src_file, dst_file)


def wav_koumura(src, dst):
    """creates a tar from the wav-koumura directory

    Parameters
    ----------
    src : pathlib.Path
        source; a sub-directory in ./results/downloads
    dst : pathlib.Path
        destination; a sub-directory in ./results/targzballs

    Returns
    -------
    None
    """
    SUBDIR = 'Bird0/Wave'
    NUM_TO_USE = 20
    exts = ['.wav']
    exts_files_map = {}
    for ext in exts:
        files_this_ext = sorted(list(
            src.joinpath(SUBDIR).glob(f'*{ext}')
        ))
        files_this_ext = files_this_ext[:NUM_TO_USE]
        exts_files_map[ext] = files_this_ext

    # hack
    exts_files_map['.xml'] = [src.joinpath('Bird0/Annotation.xml')]

    dst.mkdir()
    for _, files in exts_files_map.items():
        for src_file in files:
            dst_file = dst / src_file.name
            shutil.copy(src_file, dst_file)


def wav_textgrid(src, dst):
    """creates a tar from the wav-textgrid directory

    Parameters
    ----------
    src : pathlib.Path
        source; a sub-directory in ./results/downloads
    dst : pathlib.Path
        destination; a sub-directory in ./results/targzballs

    Returns
    -------
    None
    """
    NUM_TO_USE = 5
    wav_paths = sorted(list(
        src.joinpath('Wav Files 1').glob('*.WAV')
    ))

    textgrid_dir = src.joinpath('Textgrids')
    exts_files_map = {
        'WAV': [],
        'TextGrid': [],
    }

    num_added = 0
    wav_path_ind = 0
    while num_added < NUM_TO_USE:
        wav_path = wav_paths[wav_path_ind]
        textgrid_name = str(wav_path.name).replace('.WAV', '.TextGrid')
        textgrid_path = textgrid_dir / textgrid_name
        if textgrid_path.exists():
            exts_files_map['WAV'].append(wav_path)
            exts_files_map['TextGrid'].append(textgrid_path)
            num_added += 1
        wav_path_ind += 1
        if wav_path_ind > len(wav_paths) - 1:
            raise ValueError(
                f'was not able to find {NUM_TO_USE} .WAV files '
                f'that had matching .TextGrid files'
            )

    dst.mkdir()
    for _, files in exts_files_map.items():
        for src_file in files:
            dst_file = dst / src_file.name
            shutil.copy(src_file, dst_file)



"""
utility functions for fetching files
Adapted from MNE-Python under BSD-3 License
https://github.com/mne-tools/mne-python/blob/master/mne/utils.py#L1884
https://github.com/mne-tools/mne-python/blob/master/LICENSE.txt
"""
from math import log
from pathlib import Path
import sys
import os
import time
import urllib.request
import urllib.parse
import ftplib
import hashlib
import shutil
import tarfile
from zipfile import ZipFile
from collections.abc import Iterable

from .datasets import dataset_dict


def md5sum(fname, block_size=1048576):  # 2 ** 20
    """Calculate the md5sum for a file.
    Parameters
    ----------
    fname : str
        Filename.
    block_size : int
        Block size to use when reading.
    Returns
    -------
    hash_ : str
        The hexadecimal digest of the hash.
    """
    md5 = hashlib.md5()
    with open(fname, 'rb') as fid:
        while True:
            data = fid.read(block_size)
            if not data:
                break
            md5.update(data)
    return md5.hexdigest()


def sizeof_fmt(num):
    """Turn number of bytes into human-readable str.
    Parameters
    ----------
    num : int
        The number of bytes.
    Returns
    -------
    size : str
        The size in human-readable format.
    """
    units = ['bytes', 'kB', 'MB', 'GB', 'TB', 'PB']
    decimals = [0, 0, 1, 2, 2, 2]
    if num > 1:
        exponent = min(int(log(num, 1024)), len(units) - 1)
        quotient = float(num) / 1024 ** exponent
        unit = units[exponent]
        num_decimals = decimals[exponent]
        format_string = '{0:.%sf} {1}' % (num_decimals)
        return format_string.format(quotient, unit)
    if num == 0:
        return '0 bytes'
    if num == 1:
        return '1 byte'


class ProgressBar:
    """Generate a command-line progressbar.
    Parameters
    ----------
    max_value : int | iterable
        Maximum value of process (e.g. number of samples to process, bytes to
        download, etc.). If an iterable is given, then `max_value` will be set
        to the length of this iterable.
    initial_value : int
        Initial value of process, useful when resuming process from a specific
        value, defaults to 0.
    mesg : str
        Message to include at end of progress bar.
    max_chars : int
        Number of characters to use for progress bar (be sure to save some room
        for the message and % complete as well).
    progress_character : char
        Character in the progress bar that indicates the portion completed.
    spinner : bool
        Show a spinner.  Useful for long-running processes that may not
        increment the progress bar very often.  This provides the user with
        feedback that the progress has not stalled.
    Example
    -------
    >>> progress = ProgressBar(13000)
    >>> progress.update(3000) # doctest: +SKIP
    [.........                               ] 23.07692 |
    >>> progress.update(6000) # doctest: +SKIP
    [..................                      ] 46.15385 |
    >>> progress = ProgressBar(13000, spinner=True)
    >>> progress.update(3000) # doctest: +SKIP
    [.........                               ] 23.07692 |
    >>> progress.update(6000) # doctest: +SKIP
    [..................                      ] 46.15385 /
    """

    spinner_symbols = ['|', '/', '-', '\\']
    template = '\r[{0}{1}] {2:.05f} {3} {4}   '

    def __init__(self, max_value, initial_value=0, mesg='', max_chars=40,
                 progress_character='.', spinner=False,
                 verbose_bool=True):
        self.cur_value = initial_value
        if isinstance(max_value, Iterable):
            self.max_value = len(max_value)
            self.iterable = max_value
        else:
            self.max_value = float(max_value)
            self.iterable = None
        self.mesg = mesg
        self.max_chars = max_chars
        self.progress_character = progress_character
        self.spinner = spinner
        self.spinner_index = 0
        self.n_spinner = len(self.spinner_symbols)
        self._do_print = verbose_bool
        self.cur_time = time.time()
        self.cur_rate = 0

    def update(self, cur_value, mesg=None):
        """Update progressbar with current value of process.
        Parameters
        ----------
        cur_value : number
            Current value of process.  Should be <= max_value (but this is not
            enforced).  The percent of the progressbar will be computed as
            (cur_value / max_value) * 100
        mesg : str
            Message to display to the right of the progressbar.  If None, the
            last message provided will be used.  To clear the current message,
            pass a null string, ''.
        """
        cur_time = time.time()
        cur_rate = ((cur_value - self.cur_value) /
                    max(float(cur_time - self.cur_time), 1e-6))
        # cur_rate += 0.9 * self.cur_rate
        # Ensure floating-point division so we can get fractions of a percent
        # for the progressbar.
        self.cur_time = cur_time
        self.cur_value = cur_value
        self.cur_rate = cur_rate
        progress = min(float(self.cur_value) / self.max_value, 1.)
        num_chars = int(progress * self.max_chars)
        num_left = self.max_chars - num_chars

        # Update the message
        if mesg is not None:
            if mesg == 'file_sizes':
                mesg = '(%s / %s, %s/s)' % (
                    sizeof_fmt(self.cur_value).rjust(8),
                    sizeof_fmt(self.max_value).rjust(8),
                    sizeof_fmt(cur_rate).rjust(8))
            self.mesg = mesg

        # The \r tells the cursor to return to the beginning of the line rather
        # than starting a new line.  This allows us to have a progressbar-style
        # display in the console window.
        bar = self.template.format(self.progress_character * num_chars,
                                   ' ' * num_left,
                                   progress * 100,
                                   self.spinner_symbols[self.spinner_index],
                                   self.mesg)
        # Force a flush because sometimes when using bash scripts and pipes,
        # the output is not printed until after the program exits.
        if self._do_print:
            sys.stdout.write(bar)
            sys.stdout.flush()
        # Increment the spinner
        if self.spinner:
            self.spinner_index = (self.spinner_index + 1) % self.n_spinner

    def update_with_increment_value(self, increment_value, mesg=None):
        """Update progressbar with an increment.
        Parameters
        ----------
        increment_value : int
            Value of the increment of process.  The percent of the progressbar
            will be computed as
            (self.cur_value + increment_value / max_value) * 100
        mesg : str
            Message to display to the right of the progressbar.  If None, the
            last message provided will be used.  To clear the current message,
            pass a null string, ''.
        """
        self.update(self.cur_value + increment_value, mesg)

    def __iter__(self):
        """Iterate to auto-increment the pbar with 1."""
        if self.iterable is None:
            raise ValueError("Must give an iterable to be used in a loop.")
        for obj in self.iterable:
            yield obj
        self.update_with_increment_value(1)


def _chunk_write(chunk, local_file, progress):
    """Write a chunk to file and update the progress bar."""
    local_file.write(chunk)
    progress.update_with_increment_value(len(chunk))


def _get_ftp(url, temp_file_name, initial_size, file_size, verbose_bool):
    """Safely (resume a) download to a file from FTP."""
    # Adapted from: https://pypi.python.org/pypi/fileDownloader.py
    # but with changes

    parsed_url = urllib.parse.urlparse(url)
    file_name = os.path.basename(parsed_url.path)
    server_path = parsed_url.path.replace(file_name, "")
    unquoted_server_path = urllib.parse.unquote(server_path)

    data = ftplib.FTP()
    if parsed_url.port is not None:
        data.connect(parsed_url.hostname, parsed_url.port)
    else:
        data.connect(parsed_url.hostname)
    data.login()
    if len(server_path) > 1:
        data.cwd(unquoted_server_path)
    data.sendcmd("TYPE I")
    data.sendcmd("REST " + str(initial_size))
    down_cmd = "RETR " + file_name
    assert file_size == data.size(file_name)
    progress = ProgressBar(file_size, initial_value=initial_size,
                           max_chars=40, spinner=True, mesg='file_sizes',
                           verbose_bool=verbose_bool)

    # Callback lambda function that will be passed the downloaded data
    # chunk and will write it to file and update the progress bar
    mode = 'ab' if initial_size > 0 else 'wb'
    with open(temp_file_name, mode) as local_file:
        def chunk_write(chunk):
            return _chunk_write(chunk, local_file, progress)
        data.retrbinary(down_cmd, chunk_write)
        data.close()
    sys.stdout.write('\n')
    sys.stdout.flush()


def _get_http(url, temp_file_name, initial_size, file_size, verbose_bool):
    """Safely (resume a) download to a file from http(s)."""
    # Actually do the reading
    req = urllib.request.Request(url)
    if initial_size > 0:
        req.headers['Range'] = 'bytes=%s-' % (initial_size,)
    try:
        response = urllib.request.urlopen(req)
    except Exception:
        # There is a problem that may be due to resuming, some
        # servers may not support the "Range" header. Switch
        # back to complete download method
        logger.info('Resuming download failed (server '
                    'rejected the request). Attempting to '
                    'restart downloading the entire file.')
        del req.headers['Range']
        response = urllib.request.urlopen(req)
    total_size = int(response.headers.get('Content-Length', '1').strip())
    if initial_size > 0 and file_size == total_size:
        logger.info('Resuming download failed (resume file size '
                    'mismatch). Attempting to restart downloading the '
                    'entire file.')
        initial_size = 0
    total_size += initial_size
    if total_size != file_size:
        raise RuntimeError('URL could not be parsed properly '
                           '(total size %s != file size %s)'
                           % (total_size, file_size))
    mode = 'ab' if initial_size > 0 else 'wb'
    progress = ProgressBar(total_size, initial_value=initial_size,
                           max_chars=40, spinner=True, mesg='file_sizes',
                           verbose_bool=verbose_bool)
    chunk_size = 8192  # 2 ** 13
    with open(temp_file_name, mode) as local_file:
        while True:
            t0 = time.time()
            chunk = response.read(chunk_size)
            dt = time.time() - t0
            if dt < 0.005:
                chunk_size *= 2
            elif dt > 0.1 and chunk_size > 8192:
                chunk_size = chunk_size // 2
            if not chunk:
                if verbose_bool:
                    sys.stdout.write('\n')
                    sys.stdout.flush()
                break
            local_file.write(chunk)
            progress.update_with_increment_value(len(chunk),
                                                 mesg='file_sizes')


def _fetch_file(url, file_name, print_destination=True, resume=True,
                hash_=None, timeout=10., verbose_bool=True):
    """Load requested file, downloading it if needed or requested.
    Parameters
    ----------
    url: string
        The url of file to be downloaded.
    file_name: string
        Name, along with the path, of where downloaded file will be saved.
    print_destination: bool, optional
        If true, destination of where file was saved will be printed after
        download finishes.
    resume: bool, optional
        If true, try to resume partially downloaded files.
    hash_ : str | None
        The hash of the file to check. If None, no checking is
        performed.
    timeout : float
        The URL open timeout.
    verbose_bool : bool
        If True, send messages/events that occur during file fetching
        to standard output using print().
        Default is True.
    """
    #md5 hash should be 32 characters (hex digits, i.e. 128 bits)
    if hash_ is not None and len(hash_) != 32:
        raise ValueError('Bad hash value given, should be a 32-character '
                         'string:\n%s' % (hash_,))
    temp_file_name = file_name + ".part"
    try:
        # Check file size and displaying it alongside the download url
        u = urllib.request.urlopen(url, timeout=timeout)
        u.close()
        # this is necessary to follow any redirects
        url = u.geturl()
        u = urllib.request.urlopen(url, timeout=timeout)
        try:
            file_size = int(u.headers.get('Content-Length', '1').strip())
        finally:
            u.close()
            del u
        print('Downloading %s (%s)' % (url, sizeof_fmt(file_size)))

        # Triage resume
        if not os.path.exists(temp_file_name):
            resume = False
        if resume:
            with open(temp_file_name, 'rb', buffering=0) as local_file:
                local_file.seek(0, 2)
                initial_size = local_file.tell()
            del local_file
        else:
            initial_size = 0
        # This should never happen if our functions work properly
        if initial_size > file_size:
            raise RuntimeError('Local file (%s) is larger than remote '
                               'file (%s), cannot resume download'
                               % (sizeof_fmt(initial_size),
                                  sizeof_fmt(file_size)))
        elif initial_size == file_size:
            # This should really only happen when a hash is wrong
            # during dev updating
            warn('Local file appears to be complete (file_size == '
                 'initial_size == %s)' % (file_size,))
        else:
            # Need to resume or start over
            scheme = urllib.parse.urlparse(url).scheme
            fun = _get_http if scheme in ('http', 'https') else _get_ftp
            fun(url, temp_file_name, initial_size, file_size, verbose_bool)

        # check md5sum
        if hash_ is not None:
            print('Verifying hash %s.' % (hash_,))
            md5 = md5sum(temp_file_name)
            if hash_ != md5:
                raise RuntimeError('Hash mismatch for downloaded file %s, '
                                   'expected %s but got %s'
                                   % (temp_file_name, hash_, md5))
        shutil.move(temp_file_name, file_name)
        if print_destination is True:
            print('File saved as %s.\n' % file_name)
    except Exception:
        print('Error while fetching file %s.'
              ' Dataset fetching aborted.' % url)
        raise


def fetch(dataset_name, dst='.', remove_compressed_file=True):
    """fetch data from repositories

    Parameters
    ----------
    dataset_name : str
        name of dataset, which will be of the form:
        "audio format" + "annotation format", e.g. "wav-textgrid".
        A small example dataset of audio files with annotations in
        that format will be downloaded.
    dst : str
        Path where downloaded examples should be saved.
    remove_compressed_file : bool
        if True, remove tar.gz files after extracting.
        Default is True.

    Returns
    -------
    None

    Examples
    --------
    >>> import pollyglot
    >>> pollyglot.fetch('wav-textgrid')
    """
    try:
        dataset_meta = dataset_dict[dataset_name]
    except KeyError:
        raise KeyError('Currently there is no example dataset to download for '
                       f'{dataset_name}. To see the built-in datasets, type:\n'
                       f'>>> pollyglot.datasets.show()')
    dst = os.path.expanduser(dst)
    if not os.path.isdir(dst):
        raise NotADirectoryError(
            f'could not find destination_path: {dst}'
        )

    # for now just assume all downloads from pollyglot repository are .tar.gz
    file_name = Path(dst).joinpath(dataset_name + '.tar.gz')
    file_name = str(file_name)

    if 'md5_hash' in dataset_meta:
        md5_hash = dataset_meta['md5_hash']
    else:
        md5_hash = None

    # helpers from MNE-Python that do actual downloading.
    # Note that 'subset_url' is the url that Figshare assigns
    # the file in the pollyglot repo that contains the subset of data
    # from the original repo, that was created by pollymake
    _fetch_file(url=dataset_meta['subset_url'],
                file_name=file_name,
                hash_=md5_hash)

    if file_name.endswith('.tar.gz') or file_name.endswith('.zip'):
        print(f'extracting {file_name}')

        if file_name.endswith('.tar.gz'):
            with tarfile.open(file_name) as tar:
                tar.extractall(path=dst)

        elif file_name.endswith('.zip'):
            with ZipFile(file_name, 'r') as zipfile:
                zipfile.extractall(path=dst)

        if remove_compressed_file:
            os.remove(file_name)
